package com.dragon.system.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dragon.common.core.exception.ServiceException;
import com.dragon.common.core.utils.StringUtils;
import com.dragon.common.security.utils.SecurityUtils;
import com.dragon.system.common.domain.SysDept;
import com.dragon.system.common.domain.SysUser;
import com.dragon.system.server.mapper.SysDeptPlusMapper;
import com.dragon.system.server.service.SysDeptPlusService;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * @author 冯凯
 * @version 1.0
 * @description: 系统部门plus业务实现类
 * @date 2023/11/14 12:41
 */
@Service
public class SysDeptPlusServiceImpl extends ServiceImpl<SysDeptPlusMapper, SysDept> implements SysDeptPlusService {


    @Resource
    private SysDeptPlusMapper sysDeptPlusMapper;

    /**
     获取部门列表
     */
    @Override
    public List<SysDept> querList(SysDept dept) {
        LambdaQueryWrapper<SysDept> sysDeptLambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(dept.getDeptName())){
            sysDeptLambdaQueryWrapper.like(SysDept::getDeptName,dept.getDeptName());
        }if (StringUtils.isNotEmpty(dept.getStatus())){
            sysDeptLambdaQueryWrapper.like(SysDept::getStatus,dept.getStatus());
        }if (StringUtils.isNotEmpty(dept.getStatus())){
            sysDeptLambdaQueryWrapper.eq(SysDept::getStatus,dept.getStatus());
        }if (dept.getDeptId()!=null && dept.getDeptId()!=0){
            sysDeptLambdaQueryWrapper.eq(SysDept::getDeptId,dept.getDeptId());
        }if (dept.getParentId()!=null && dept.getParentId()!=0){
            sysDeptLambdaQueryWrapper.eq(SysDept::getParentId,dept.getParentId());
        }
        Object dataScope = dept.getParams().get("params.dataScope");
        System.out.println(dataScope);
        if (Objects.nonNull(dataScope) && dataScope instanceof String dataScopeData)
        sysDeptLambdaQueryWrapper.apply(dataScopeData);
        sysDeptLambdaQueryWrapper.eq(SysDept::getDelFlag,"0");
        sysDeptLambdaQueryWrapper.orderByAsc(SysDept::getParentId,SysDept::getOrderNum);
        return this.list(sysDeptLambdaQueryWrapper);
    }


    /**
     * 根据部门编号获取详细信息
     */
    @Override
    public void checkDeptDataScope(Long deptId) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
            SysDept dept = new SysDept();
            dept.setDeptId(deptId);
            List<SysDept> depts = this.querList(dept);
            if (StringUtils.isEmpty(depts)) {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }

    /**
     检验部门名称的唯一性
     */
    @Override
    public boolean checkDeptNameUnique(SysDept dept) {
        LambdaQueryWrapper<SysDept> sysDeptLambdaQueryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(dept.getDeptName(),"部门名称不能为空");
        Assert.notNull(dept.getParentId(),"父级id不能为空");
        sysDeptLambdaQueryWrapper.eq(SysDept::getDeptName,dept.getDeptName());
        sysDeptLambdaQueryWrapper.eq(SysDept::getParentId,dept.getParentId());
        sysDeptLambdaQueryWrapper.eq(SysDept::getDelFlag,"0");
        SysDept one = this.getOne(sysDeptLambdaQueryWrapper);
        return one!=null?true:false;
    }

    @Override
    public int updateDept(SysDept dept) {
        SysDept newParentDept = sysDeptPlusMapper.selectById(dept.getDeptId());
        SysDept oldDept = sysDeptPlusMapper.selectById(dept.getParentId());
        if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept)) {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
        }
        return 0;
    }

    @Override
    public boolean hasChildByDeptId(Long deptId) {
        LambdaQueryWrapper<SysDept> sysDeptLambdaQueryWrapper = new LambdaQueryWrapper<>();
        sysDeptLambdaQueryWrapper.eq(SysDept::getParentId,deptId);
        sysDeptLambdaQueryWrapper.eq(SysDept::getDelFlag,"0");
        return count(sysDeptLambdaQueryWrapper)>0;
    }

    @Override
    public boolean checkDeptExistUser(Long deptId) {
        int result=sysDeptPlusMapper.checkDeptExistUser(deptId);
        return result>0;
    }

    @Override
    public int selectNormalChildrenDeptById(Long deptId) {
        return sysDeptPlusMapper.selectNormalChildrenDeptById(deptId);
    }

    private void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors) {
        List<SysDept> children =  sysDeptPlusMapper.selectChildrenDeptById(deptId);
        for (SysDept child : children) {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (children.size() > 0) {
            sysDeptPlusMapper.updateDeptChildren(children);
        }
    }
}
